<?php

/**
 * CommerceML v2.03 parser class
 */
class nc_netshop_cml2parser {

  var $dataValue, $dataIndex, $parser, $encoding;
  var $source_id, $catalogue_id, $everything_clear, $main_xml_array;

  function nc_netshop_cml2parser (&$db, &$_UTFConverter, $source_id, $catalogue_id, $filename) {
    // set variables
    $this->db = $db;
    $this->uniconv = $_UTFConverter;
    $this->filename = $filename;
    $this->filename_path = $GLOBALS['TMP_FOLDER'].$filename;
    $this->source_id = (int)$source_id;
    $this->catalogue_id = (int)$catalogue_id;
    $this->everything_clear = true;
    $this->not_mapped_sections = 0;
    $this->main_xml_array = array();

    // Load currencies, units
    $res = $this->db->get_results("SELECT `ShopCurrency_ID`, `ShopCurrency_Name` FROM `Classificator_ShopCurrency`", ARRAY_A);
    foreach ($res AS $value) {
      $this->currency[$value['ShopCurrency_Name']] = $value['ShopCurrency_ID'];
    }
    $res = $this->db->get_results("SELECT `ShopUnits_ID`, `ShopUnits_Name` FROM `Classificator_ShopUnits`", ARRAY_A);
    foreach ($res AS $value) {
      $this->units[$value['ShopUnits_Name']] = $value['ShopUnits_ID'];
    }

    // Get list of goods templates
    $this->shop_classes = $this->db->get_results("SELECT DISTINCT c.`Class_ID` AS id, c.`Class_Name` AS name
      FROM `Class` AS c, `Field` AS f
      WHERE c.`Class_ID` = f.`Class_ID`
      AND f.`Field_Name` LIKE 'Price%'
      AND ( c.`Class_Group` = 'Netshop' OR c.`Class_Group` = 'Интернет-магазин' )
      ORDER BY c.`Class_ID`", ARRAY_A);
  }

  function parse_importing_file () {
    // file existence
    if ( !file_exists($this->filename_path) ) return false;
    //ini_set("memory_limit", "32M");
    // create parser ("UTF-8")
    $this->parser = xml_parser_create();
    // as object
    xml_set_object($this->parser, $this);
    // default encoding = encoding in xml_parser_create()
    xml_parser_set_option($this->parser, XML_OPTION_TARGET_ENCODING, "UTF-8");
    // tag names in lowercase, important!
    xml_parser_set_option($this->parser, XML_OPTION_CASE_FOLDING, false);
    // open/close tags
    xml_set_element_handler($this->parser, "tag_open", "tag_closed");
    // pdata/cdata tags string
    xml_set_character_data_handler($this->parser, "tag_data");

    $this->import_ignore_tags = array("ЗначенияРеквизитов", "СтавкиНалогов");

    $f = fopen($this->filename_path, "r");
    // find
    if ( function_exists("ftell") && function_exists("rewind") ) {
      $file_end = filesize($this->filename_path);
      rewind($f);
      // show progress bar
      echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_XML_FILE."</b><br/>\r\n";
      $this->progress_bar_show("parse_progress");
      echo "<br/>\r\n";
    }

    // get file info
    if ( function_exists("fstat") ) {
      $file_stat = fstat($f);
      $file_block_size = $file_stat['blksize'];
    }
    // file block size
    if ($file_block_size <= 0) $file_block_size = 8192;

    while ( $xml_string = fread($f, $file_block_size) ) {
      $this->xml_data = xml_parse( $this->parser, $xml_string, feof($f) );
      if ($file_end > 0) {
        // procents completed
        $percent = intval( ftell($f) /$file_end * 100 );
        $this->progress_bar_update("parse_progress", $percent);
      }
    }
    fclose($f);

    // show error
    if (!$this->xml_data) {
      sprintf(
        "XML parsing error: %s at line %d",
        xml_error_string( xml_get_error_code($this->parser) ),
        xml_get_current_line_number($this->parser)
      );
    }
    // clear parser
    xml_parser_free($this->parser);
  }

  function tag_open($parser, $name, $attrs) {
    $_name = $this->uc($name);
    // ignore this tag
    if ($_name=="КоммерческаяИнформация") return;
    // ignore this tags with ignore descriptor
    if ( in_array($_name, $this->import_ignore_tags) ) $this->tag_ignore = true;
    // ignore section
    if ($this->tag_ignore) return;
    // associate main arrays
    if (!$this->tag_level) {
      switch ($_name) {
        case "Классификатор":
          $this->main_xml_array = "classifier";
        break;
        case "Каталог":
          $this->main_xml_array = "catalogue";
        break;
        case "ПакетПредложений":
          $this->main_xml_array = "offers";
        break;
      }
      $this->this->main_xml_array = array();
    }
    // set level up
    $this->tag_level++;
    // append to main array
    array_push( $this->this->main_xml_array, array('n' => $_name) );
  }

  function tag_data($parser, $tag_data) {
    // ignore section
    if ( trim($tag_data)=="" || $this->tag_ignore ) return;
    // index value
    $arr_index = count($this->this->main_xml_array) - 1;
    // append data in array
    if( isset($this->this->main_xml_array[$arr_index]['d']) ) {
      $this->this->main_xml_array[$arr_index]['d'].= $this->uc($tag_data);
    }
    else {
      $this->this->main_xml_array[$arr_index]['d'] = $this->uc($tag_data);
    }
  }

  function tag_closed($parser, $name) {
    $_name = $this->uc($name);
    // ignore this tag
    if ($_name=="КоммерческаяИнформация") return;
    // ignore this tags with ignore descriptor
    if ( in_array($_name, $this->import_ignore_tags) ) {
      $this->tag_ignore = false;
      return;
    }
    // ignore section
    if ($this->tag_ignore) return;
    // set level down
    $this->tag_level--;
    $arr_index = count($this->this->main_xml_array) - 1;
    // if child - replace element in array
    if ( $arr_index > 0 && $this->tag_level!=0 ) {
      $append = array_pop($this->this->main_xml_array);
      $this->this->main_xml_array[$arr_index - 1]['c'][] = $append;
    }
    // get_data
    if (!$this->tag_level) {
      $_xml_array = array();
      switch ($_name) {
        case "Классификатор":
          $_xml_array = array_shift($this->this->main_xml_array);
          $this->get_classifier_data("Группы", "Группа", "Ид", $_xml_array['c']);
        break;
        case "Каталог":
          $_xml_array = array_shift($this->this->main_xml_array);
          $this->get_catalogue_data("Товары", "Товар", $this->import_ignore_tags, $_xml_array['c']);
        break;
        case "ПакетПредложений":
          $_xml_array = array_shift($this->this->main_xml_array);
          $this->get_offers_data($_xml_array['c']);
        break;
      }
      unset ($_xml_array);
    }
  }

  function get_templates () {
    // check templates in cache
    //if ( file_exists($this->filename_path.".templates.cache") ) {
    //  return $this->cache_data_out("templates");
    //}
    // data templates (classes)
    $sub_ids = array();
    // get data
    $this->sub_structure = $this->cache_data_out("sub_structure");
    // if template is unknown, get it
    if ($this->sub_structure) {
      foreach ($this->sub_structure AS $value) {
        if ($value['Subdivision_ID']) $sub_ids[] = $value['Subdivision_ID'];
      }
    }
    else {
      $sub_ids = $this->db->get_col("SELECT `value`
        FROM `Netshop_ImportMap`
        WHERE `source_id` = '".$this->source_id."'
        AND `type` = 'section'");
    }

    if ( !empty($sub_ids) ) {
      $res = $this->db->get_results("SELECT `Subdivision_ID`, `Class_ID`, `Sub_Class_ID`
        FROM `Sub_Class`
        WHERE `Subdivision_ID` IN (".join(",", $sub_ids).")
        ORDER BY `Priority` DESC", ARRAY_A);
      foreach ($res AS $value) {
          $templates[ $value['Subdivision_ID'] ]['class_id'] = $value['Class_ID'];
          $templates[ $value['Subdivision_ID'] ]['subclass_id'] = $value['Sub_Class_ID'];
      }
    }

    // cache file
    $this->cache_data_in($templates, "templates");
    // return result
    return $templates;
  }


  /**
   * Update source in base
   * @return true if all good
   */
  function update_sources () {
    // get data
    $this->catalogue_data_properties = $this->cache_data_out("catalogue_data_properties");
    // return if no data found
    if ( empty($this->catalogue_data_properties) ) return false;

    // get properties
    $this_id = $this->catalogue_data_properties["Ид"];
    $this_classificator_id = $this->catalogue_data_properties["ИдКлассификатора"];
    $this_name = trim( $this->catalogue_data_properties["Наименование"] );
    // update sources
    $result = $this->db->query("UPDATE `Netshop_ImportSources` SET `external_id` = '".$this_classificator_id." ".$this_id."' WHERE `source_id` = '".$this->source_id."'");
    // destroy variables
    unset ($this->catalogue_data_properties);
    // true or false
    return $result ? true : false;
  }


  /**
   * Put cache file in temp directory /netcat/tmp/
   * @param array to serialise
   * @param string suffix for file name
   * @return bytes in cached file or false
   */
  function cache_data_in (&$arr, $suffix) {
    $_path = $this->filename_path.".".$suffix.".cache";
    // make serialise
    $cached = serialize($arr);
    unset ($arr);
    $f = fopen($_path, "w");
    $bytes_writed = fwrite($f, $cached);
    unset ($cached);
    fclose($f);
    // return bytes in file or false
    return $bytes_writed;
  }

  /**
   * Get array from cache file in temp directory /netcat/tmp/
   * @param string suffix for file name
   * @return unserialise array or false
   */
  function cache_data_out ($suffix, $delete = false) {
    $_path = $this->filename_path.".".$suffix.".cache";
    // check file existence
    if ( !file_exists($_path) ) return false;
    // get contents from file
    $data = file_get_contents($_path);
    // get array from serialise data
    $arr = unserialize($data);
    // unset variable
    unset ($data);
    // delete file if set descriptor
    if ($delete) unlink($_path);
    // return return array or false
    return !empty($arr) ? $arr : false;
  }

  /**
   * Delete cached file(s) in temp directory /netcat/tmp/
   * @param mixed file(s) to delete
   * @return true or false
   */
  function cache_data_destroy ($arr) {
    // convert to array
    $arr = (array)$arr;
    $file_count = 0;
    foreach ($arr AS $data_file) {
      $_path = $this->filename_path.".".$data_file.".cache";
      if ( file_exists($_path) ) {
        if ( unlink($_path) ) $file_count++;
      }
    }
    // return result
    return (count($arr) - $file_count) ? false : true;
  }

  /**
   * Check cached file(s) in temp directory /netcat/tmp/
   * @return true if any file cached or false
   */
  function cache_data_exist ($clear = false) {
    // check
    $arr = array("classifier_data", "offers_data_properties", "offers_data_price_type", "offers_data_offers", "catalogue_data_properties", "catalogue_data_commodity", "catalogue_data_commodity_fields", "offers_data_offers_fields", "sub_structure", "offers_data_packets_arr", "templates", "map_groups", "new_group");
    $file_count = 0;
    foreach ($arr AS $data_file) {
      $_path = $this->filename_path.".".$data_file.".cache";
      if ( file_exists($_path) ) {
        if (!$clear) {
          $file_count++;
        }
        else {
          if ( unlink($_path) ) $file_count++;
        }
      }
    }
    // return result
    return $file_count ? true : false;
  }


  /**
   * Get classifier data function
   * make flattern group array with parent id
   */
  function get_classifier_data ($plural_tag, $single_tag, $id_tag, $arr, $parent = array(), $level = 0) {
    // walking...
    foreach ($arr AS $key => $value) {
      // if plural tag - call recursive function
      if ($value['n']==$plural_tag) {
        $this->get_classifier_data($plural_tag, $single_tag, $id_tag, $value['c']);//, $id
        continue;
      }

      // clearing parent array to current level
      if ( !empty($parent) ) {
        for ($i = count($parent) - 1; $i >= $level; $i--) {
          array_pop($parent);
        }
      }

      // if single tag - put data in flattern array
      if ($value['n']==$single_tag) {
        $index = count($this->classifier_data) + 1;
        // if set parent put it in flattern array
        $this->classifier_data[$index]['parent_group_id'] = $parent ? $parent[$level - 1] : 0;
        foreach ($value['c'] AS $child) {
          // if set embedded groups
          if ( $child['n']==$plural_tag) {
            $this->get_classifier_data($plural_tag, $single_tag, $id_tag, $child['c'], $parent, $level + 1);
            continue;
          }
          // put data in flattern array
          $this->classifier_data[$index][$child['n']] = $child['d'];
          if ( $child['n']==$id_tag ) $parent[$level] = $child['d'];
        }
      }
    }
    // cache file
    $this->cache_data_in($this->classifier_data, "classifier_data");
  }




  function import_classifier_data () {
    // get data
    $this->classifier_data = $this->cache_data_out("classifier_data");
    $this->sub_structure = $this->cache_data_out("sub_structure");

    // return if no data found
    if ( empty($this->classifier_data) ) return false;

    $this->shop = GetSubdivisionByType($GLOBALS["MODULE_VARS"]["netshop"]["SHOP_TABLE"], "Subdivision_ID, Subdivision_Name", $this->catalogue_id);

    $parent_sub_id = $this->shop["Subdivision_ID"];

    // after map_sections_dialog(), this arrays is accessible
    if ( isset($_POST['map_groups']) ) {
      $this->map_groups = $_POST['map_groups'];
      // cache file
      $this->cache_data_in($this->map_groups, "map_groups");
    }
    else {
      // get data
      $this->map_groups = $this->cache_data_out("map_groups");
    }
    if ( isset($_POST['new_group']) ) {
      $this->new_group = $_POST['new_group'];
      // cache file
      $this->cache_data_in($this->new_group, "new_group");
    }
    else {
      // get data
      $this->new_group = $this->cache_data_out("new_group");
    }

    $current_num = 1;
    $total_objects = count($this->classifier_data);
    foreach ($this->classifier_data AS $key => $value) {

      if ($current_num==1 && $this->map_groups) {
        echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_CATALOGUE_STRUCTURE."</b><br/>\r\n";
        $this->progress_bar_show("structure_progress");
        echo "<br/>\r\n";
      }

      $parent_group_id = $value['parent_group_id'];
      $group_id = $value['Ид'];
      $name = $value['Наименование'];
      $name = trim($name);

      if ($this->map_groups[$group_id]) $template_id = (int)$this->new_group[$group_id]["template"];

      switch (true) {
        // ignore this group
        case ($this->map_groups[$group_id] == -1):
          $sub_id = -1;
          $parent_sub_id = isset($parent_group_id) && $this->sub_structure[$parent_group_id] ? $this->sub_structure[$parent_group_id]["Subdivision_ID"] : $this->shop["Subdivision_ID"];
        break;
        // new group posted
        case ($this->map_groups[$group_id]=="new"):
          // если у раздела есть родители в дзен-массиве то ставим соответствие с Subdivision_ID из массива $parent
          $parent_sub_id = isset($parent_group_id) && $this->sub_structure[$parent_group_id] ? $this->sub_structure[$parent_group_id]["Subdivision_ID"] : $this->shop["Subdivision_ID"];

          // заменяем любой символ, не образующий "слово", и ставим заглавные буквы в начале каждого слова
          $english_name = nc_preg_replace("/\W+/", "", ucwords( nc_transliterate($name) ) );

          if (!$parent_data[$parent_sub_id]) {
            $parent_data[$parent_sub_id] = $this->db->get_row("SELECT * FROM `Subdivision` WHERE `Subdivision_ID` = '".$parent_sub_id."'", ARRAY_A);
          }

          $priority = (int)$this->db->get_var("SELECT MAX(`Priority`) + 1 FROM `Subdivision` WHERE `Parent_Sub_ID` = '".$parent_sub_id."'");

          // если EnglishName повторяется, добавляем дополнительный индекс
          $english_name_suffix = "";
          while ( $this->db->get_var("SELECT COUNT(*) FROM `Subdivision` WHERE `Parent_Sub_ID` = '".$parent_sub_id."' AND `EnglishName` = '".$english_name.$english_name_suffix."'") ) {
            $english_name_suffix += 1;
          }
          $english_name.= (string)$english_name_suffix;

          $hidden_url = $parent_data[$parent_sub_id]['Hidden_URL'].$english_name."/";

          // добавляем запись в базу
          $this->db->query("INSERT INTO `Subdivision`
            SET `Catalogue_ID` = '".$this->catalogue_id."',
            `Parent_Sub_ID` = '".$parent_sub_id."',
            `Subdivision_Name` = '".$this->db->escape($name)."',
            `Template_ID` = 0,
            `EnglishName` = '".$english_name."',
            `LastUpdated` = NOW(),
            `Created` = NOW(),
            `Hidden_URL` = '".$hidden_url."',
            `Priority` = '".$priority."',
            `Checked` = 1");
          // Subdivision_ID
          $sub_id = $this->db->insert_id;

          $this->db->query("INSERT INTO Sub_Class
            SET `Subdivision_ID` = '".$sub_id."',
            `Class_ID` = '".$template_id."',
            `Sub_Class_Name` = '".$this->db->escape($name)."',
            `EnglishName` = '".$english_name."',
            `Priority` = 0,
            `Checked` = 1,
            `Catalogue_ID` = '".$this->catalogue_id."',
            `DefaultAction` = 'index',
            `Created` = NOW(),
            `LastUpdated` = NOW()");

          $sub_class_id = $this->db->insert_id;

          // import map
          $this->db->query("REPLACE INTO `Netshop_ImportMap`
            SET `source_id` = '".$this->source_id."',
            `type` = 'section',
            `source_string` = '".$this->db->escape($group_id)."',
            `value` = '".$sub_id."'");
        break;

        case ( int($this->map_groups[$group_id]) ):
          $sub_id = (int)$this->map_groups[$group_id];
          // Sub_Class_ID value for $this->sub_structure array
          $sub_class_id = $this->db->get_var("SELECT `Sub_Class_ID` FROM `Sub_Class` WHERE `Class_ID` = '".$template_id."' AND `Subdivision_ID` = '".$sub_id."'");
          // указано соответствие
          $this->db->query("REPLACE INTO `Netshop_ImportMap`
            SET `source_id` = '".$this->source_id."',
            `type` = 'section',
            `source_string` = '".$this->db->escape($group_id)."',
            `value` = '".$sub_id."'");
        break;

        default:
          // Найти соответствие разделу (по внешнему идентификатору)
          $sub_id = $this->db->get_var("SELECT m.`value`
            FROM `Netshop_ImportMap` AS m, Subdivision AS s
            WHERE m.`type` = 'section'
            AND m.`source_string` = '".$this->db->escape($group_id)."'
            AND m.`source_id` = '".$this->source_id."'
            AND m.`value` = s.`Subdivision_ID`
            ORDER BY m.`source_id` = '".$this->source_id."' DESC");
          // спросить потом
          if (!$sub_id) $this->not_mapped_sections++;
      }

      $this->sub_structure[$group_id] = array(
        // imperative
        "Subdivision_ID" => $sub_id,
        "Sub_Class_ID" => $sub_class_id,
        // optional for default and selected compliance Subdivision
        "Parent_Sub_ID" => $parent_sub_id,
        "Name" => $name
      );

      // procents completed
      $percent = intval($current_num/$total_objects * 100);
      $this->progress_bar_update("structure_progress", $percent);
      // increment
      $current_num++;
    }
    // destroy variable
    unset ($this->classifier_data);

    // cache file
    if ( !empty($this->sub_structure) ) {
      $this->cache_data_in($this->sub_structure, "sub_structure");
      return true;
    }

    return false;
  }


  function map_sections_dialog () {

    if (!$silent_1c_import) {

      echo "<style type='text/css'>";
      echo ".divadd {border:1px solid #DDD; background-color:#F0F0F0; padding:3px;}";
      echo "select {width:auto}";
      echo "</style>";

      echo "<script type='text/javascript'>";
      echo "function switch_divadd(gid) {";
      echo "  var sel = document.getElementById('map_groups' + gid);";
      echo "  var val = sel.options[sel.selectedIndex].value;";
      echo "  document.getElementById('divadd' + gid).style.display = (val=='new' ? '' : 'none');";
      echo "}";
      echo "</script>";

      $this->everything_clear = false;

      $sections = GetStructure($this->shop["Subdivision_ID"], "Checked = 1");

      $sections_as_options = "";
      foreach ($sections AS $row) {
        $sections_as_options.= "<option value='".$row['Subdivision_ID']."'>".str_repeat("&nbsp;", ($row["level"] + 1) * 4).$row['Subdivision_Name']."</option>\r\n";
      }

      // Ask about groups we don't know
      echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_MAP_SECTION.":</b>\r\n";
      echo "<table border='0' cellspacing='8' cellpadding='0'>\r\n";

      $templates_count = count($this->shop_classes);
      $this->goods_template_ids = array();

      $templates_as_options = "";
      // netshop goods classes
      foreach ($this->shop_classes AS $value) {
        $templates_as_options.= "<option value=".$value['id'].(!$templates_as_options ? " selected" : "").">".$value['name']."</option>\r\n";
        $this->goods_template_ids[] = $value['id'];
      }

      // get data
      $this->sub_structure = $this->cache_data_out("sub_structure");
      // group -> sub list
      if ( is_array($this->sub_structure) && !empty($this->sub_structure) ) {
        foreach ($this->sub_structure AS $gid => $group) {
          if (!$group['Subdivision_ID']) {
            $parent = $group['Parent_Sub_ID'];
            if (!$parent) $parent = "[root]";
            echo "<tr valign=top>";
            echo "<td title='".$gid." &larr; ".$parent."'>".$group['Name']."</td>";
            echo "<td>&rarr;</td>";
            echo "<td>";
            echo "<select name='map_groups[".$gid."]'".($templates_count > 1 ? " onchange='switch_divadd(\"".$gid."\")'" : "")." id='map_groups".$gid."'>";
            echo "<option value='new' style='color:navy'>".NETCAT_MODULE_NETSHOP_IMPORT_CREATE_SECTION.($templates_count > 1 ? " &nbsp; &darr; &nbsp;" : "")."</option>";
            echo "<option value='-1'>".NETCAT_MODULE_NETSHOP_IMPORT_IGNORE_SECTION."</option>";
            echo "<option value='-1'>----------------------------------------</option>";
            echo $sections_as_options;
            echo "</select>";
            echo "</td>";
            echo "<td>";
            echo "<div class='divadd' id='divadd".$gid."'".($templates_count==1 ? " style='display:none'" : "").">";
            echo NETCAT_MODULE_NETSHOP_IMPORT_TEMPLATE.":&nbsp;<select name='new_group[".htmlspecialchars($gid)."][template]'>\r\n";
            echo $templates_as_options;
            echo "</select>";
            echo "</div>";
            echo "</td>";
            echo "</tr>";
          }
        }
      }

      echo "</table><br/>";
    }

  }


  /**
   * This function parse offers section
   * @return array($properties, $price_type, $offers) - parsed data
   */
  function get_offers_data ($arr) {
    // walking...
    foreach ($arr AS $key => $value) {
      // offers properties
      // if ( $value['n'] && $value['d'] ) {
        // $this->offers_data_properties[$value['n']] = $value['d'];
        // continue;
      // }
      // currencies type
      if ( $value['n']=="ТипыЦен" ) {
        foreach ($value['c'] AS $child) {
          if ( $child['n']=="ТипЦены" ) {
            $index_price = count($this->offers_data_price_type);
            foreach ($child['c'] AS $price_type_data) {
              if ($price_type_data['d']) {
                $this->offers_data_price_type[$index_price][$price_type_data['n']] = $price_type_data['d'];
              }
              ### // if ( $price_type_data['name']==$this->cu("Налог") ) {                // $tax_name = "";                // $tax_value = 0;                // foreach ($price_type_data['children'] AS $tax_rate_data) {                  // if ( $tax_rate_data['name']==$this->cu("Наименование") ) $tax_name = $tax_rate_data['data'];                  // if ( $tax_rate_data['name']==$this->cu("УчтеноВСумме") ) $tax_value = $tax_rate_data['data'];                // }                // // make array( 0 => array(tax_name => tax_value), 1 => ...)                // if ($tax_name && $tax_value) {                  // $this->offers_data_price_type[$index_price][$price_type_data['name']] = array($tax_name => $tax_value);                // }              // }
            }
          }
        }
        // cache file
        $this->cache_data_in($this->offers_data_price_type, "offers_data_price_type");
        continue;
      }
      // offers packets
      if ( $value['n']=="Предложения" ) {
        foreach ($value['c'] AS $child) {
          if ( $child['n']=="Предложение" ) {

            $offers_index = count($this->offers_data_offers);
            foreach ($child['c'] AS $offers_data) {

              if ($offers_data['n'] && $offers_data['d']!==0) {
                $this->offers_data_offers[$offers_index][$offers_data['n']] = $offers_data['d'];
                $this->offers_data_offers_fields[$offers_data['n']]++;
              }

              if ( $offers_data['n']=="Цены" ) {
                foreach ($offers_data['c'] AS $currencies) {
                  if ( $currencies['n']=="Цена" ) {
                    $curr_index = count($this->offers_data_offers[$offers_index][$currencies['n']]);
                    foreach ($currencies['c'] AS $curr_data) {
                      $this->offers_data_offers[$offers_index][$currencies['n']][$curr_index][$curr_data['n']] = $curr_data['d'];
                      //$this->offers_data_offers_fields[$curr_data['name']]++;
                    }
                  }
                }
              }
            }
          }
        }
        // cache files
        $this->cache_data_in($this->offers_data_offers, "offers_data_offers");
        $this->cache_data_in($this->offers_data_offers_fields, "offers_data_offers_fields");
      }
    }
  }


  function import_offers_data () {
    // clear?
    //if (!$this->everything_clear) return false;
    // get data
    $this->offers_data_price_type = $this->cache_data_out("offers_data_price_type");
    // return if no data found
    if ( empty($this->offers_data_price_type) ) return false;

    if (!$this->templates) $this->templates = $this->get_templates();

    // import price type
    $packets = array();
    foreach ($this->offers_data_price_type AS $key => $value) {

      $offers_id = $value['Ид'];
      $name = $value['Наименование'];
      $name = trim($name);

      $packets[$offers_id]["name"] = $name;
      $map_packet = $_POST['map_packets'][urlencode($offers_id)];

      // write compliance (second pass)
      if ($map_packet) {
        $this->db->query("REPLACE INTO `Netshop_ImportMap`
          SET `source_id` = '".$this->source_id."',
          `type` = 'price',
          `source_string` = '".$this->db->escape($offers_id)."',
          `value` = '".$this->db->escape($map_packet)."'");
      }
    }
    // destroy variable
    unset ($this->offers_data_price_type);

    // destroy old file, information may be updated
    $this->cache_data_destroy("offers_data_packets_arr");
    // count packets from base for "price" type
    if ( !empty($packets) ) {
      $packets_res = $this->db->get_results("SELECT `source_string` AS id, value
        FROM `Netshop_ImportMap`
        WHERE `source_id` = '".$this->source_id."'
        AND `type` = 'price'
        AND `source_string` IN ('".join("','", array_keys($packets))."')", ARRAY_A);
      $packets_from_base = $this->db->num_rows;
    }
    // mapped packets
    if ( !empty($packets_res) ) {
      foreach ($packets_res AS $value) {
        $packets[$value['id']]["column"] = $value['value'];
      }
    }
    // not_mapped_packets value
    $this->not_mapped_packets = count($packets) - $packets_from_base;
    // for map_packets_dialog function
    // cache files
    $this->cache_data_in($packets, "offers_data_packets_arr");

    // get data from cache file
    $this->offers_data_offers_fields = $this->cache_data_out("offers_data_offers_fields");
    // action)
    if ( !empty($this->offers_data_offers_fields) ) {
      $need_tags = array("Количество");
      foreach ($this->offers_data_offers_fields AS $xml_tag => $tag_count) {
        $xml_tag = trim($xml_tag);
        //echo $xml_tag."<br/>";
        if ( !in_array($xml_tag, $need_tags) ) continue;

        $fields[$xml_tag]['name'] = $xml_tag;
        foreach ($this->shop_classes AS $class) {
          $map_field = $_POST['map_fields'][ $class['id'] ][ urlencode($xml_tag) ];

          // write compliance (second pass)
          if ($map_field) {
            $this->db->query("REPLACE INTO `Netshop_ImportMap`
              SET `source_id` = '".$this->source_id."',
              `type` = 'property',
              `source_string` = '".$this->db->escape($xml_tag)."',
              `value` = '".$this->db->escape($map_field)."'");
          }
        }
      }
      // destroy variable
      unset ($this->offers_data_offers_fields);
    }
    // count fields from base for "property" type
    if ( !empty($fields) ) {
      $fields_res = $this->db->get_results("SELECT `source_string` AS id, `value`
        FROM `Netshop_ImportMap`
        WHERE `source_id` = '".$this->source_id."'
        AND `type` = 'property'
        AND `source_string` IN ('".join("','", array_keys($fields))."')", ARRAY_A);
      $fields_from_base = $this->db->num_rows;
    }
    // mapped fields
    if ( !empty($fields_res) ) {
      foreach ($fields_res AS $value) {
        $fields[ $value['id'] ]['column'] = $value['value'];
      }
    }
    // not_mapped_fields value
    $this->not_mapped_fields = count($fields) - $fields_from_base;
    // for map_fields_dialog function
    $this->not_mapped_fields_arr = is_array($this->not_mapped_fields_arr) ? array_merge($this->not_mapped_fields_arr, $fields) : $fields;

    // if not_mapped values - return and call dialog(s)
    if ($this->not_mapped_packets || $this->not_mapped_fields || !$this->everything_clear) return false;

    // count components in this source
    foreach ($this->templates AS $data) {
      $messages_tbl[$data["class_id"]] = "`Message".$data["class_id"]."`";
      $this_class = $data["class_id"];
    }

    $current_num = 1;
    // get data
    $this->offers_data_offers = $this->cache_data_out("offers_data_offers");
    $total_objects = count($this->offers_data_offers);
    // action)
    foreach ($this->offers_data_offers AS $key => $value) {

      if ($current_num==1) {
        echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_OFFERS."</b><br/>\r\n";
        $this->progress_bar_show("offers_progress");
        echo "<br/>\r\n";
      }

      $this_id = $value['Ид'];
      $this_name = trim( $value['Наименование'] );
      $this_currency_arr = $value['Цена'];
      // if components in source > 1,
      //check commodity in all components
      if (!empty($messages_tbl) && count($messages_tbl) > 1) {
        $sub_id = $this->db->get_var("SELECT `Subdivision_ID`
          FROM ".join(", ", $messages_tbl)."
          WHERE `ItemID` = '".$this_id."'
          AND `ImportSourceID` = '".$this->source_id."'");
        $this_class = $this->templates[$sub_id]["class_id"];
      }

      $this_currencies = array();
      foreach ($this_currency_arr AS $object_currency) {
        $this_currencies[ $object_currency['ИдТипаЦены'] ] = $object_currency;
      }

      // commodity currency
      foreach ($packets AS $packet_id => $packet) {

        if (!$packet["column"] || $packet["column"]==-1) continue;
        // get price
        $this_prop[ $packet["column"] ] = $this_currencies[$packet_id]["ЦенаЗаЕдиницу"];
        // get currency
        $this_currency = $this_currencies[$packet_id]["Валюта"];

        if ( $this_currency=="руб" ) $this_currency = "RUR";

        // check if currency exists, else add it
        if (!$this_currency) {
           $this_currency = "RUR";
        }
        else if (!$this->currency[$this_currency]) {
          $this->db->query("INSERT INTO `Classificator_ShopCurrency` SET `ShopCurrency_Name` = '".$this_currency."'");
          $this->currency[$this_currency] = $this->db->insert_id;
        }
        // !!! TODO: check (a) price column exists in template
        $currency_add = str_replace("Price", "", $packet["column"]);
        $this_prop["Currency".$currency_add] = $this->currency[$this_currency];

      }

      // additional fields compile
      $filetable_lastid = 0;
      if ( !empty($fields) ) {
        foreach ($fields AS $field_key => $field_data) {
          if ($field_data['column']==-1) continue;
          list($field_name, $field_type) = $this->db->get_row("SELECT `Field_Name`, `TypeOfData_ID` FROM `Field`
            WHERE `Field_ID` = '".(int)$field_data['column']."'", ARRAY_N);
          $this_prop[$field_name] = trim( $value[$field_key] );
        }
      }

      // collect fields in temp array
      if ( !empty($this_prop) ) {
        $query = array();
        foreach ($this_prop AS $k => $v) {
          $query[] = "`".$k."` = '".$this->db->escape($v)."'";
        }
        // equip MySQL append query
        $query_str = "SET ".join(", ", $query);
        unset ($query);
      }

      // update object
      $this->db->query("UPDATE `Message".$this_class."` ".$query_str." WHERE `ItemID` = '".$this_id."' AND `ImportSourceID` = '".$this->source_id."'");

      // procents completed
      $percent = intval($current_num/$total_objects * 100);
      $this->progress_bar_update("offers_progress", $percent);
      // increment
      $current_num++;
    }
    // destroy variable
    unset ($this->offers_data_offers);
  }


  function map_packets_dialog () {

    if ($silent_1c_import) return false;

    $this->everything_clear = false;

    echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_MAP_PRICE.":</b>\r\n";
    echo "<table border='0' cellspacing='8' cellpadding='0'>\r\n";

    $num_goods_types = $this->db->get_var("SELECT COUNT(DISTINCT(c.`Class_ID`))
      FROM `Field` AS f, `Class` AS c
      WHERE f.`Field_Name` LIKE 'Price%'
      AND c.`Class_ID`= f.`Class_ID`
      AND c.`Class_Group` = 'Netshop'");

    $res = $this->db->get_results("SELECT f.`Field_Name` AS id, f.`Description` AS name, COUNT(f.`Field_Name`) AS cnt
      FROM `Class` AS c, `Field` AS f
      WHERE c.`Class_ID` = f.`Class_ID`
      AND f.`Field_Name` LIKE 'Price%'
      AND c.`Class_Group` = 'Netshop'
      GROUP BY f.`Field_Name`
      HAVING cnt = '".$num_goods_types."'", ARRAY_A);

    $price_col_options = "";
    if ( !empty($res) ) {
      foreach ($res AS $value) {
        $price_col_options.= "<option value='".$value['id']."'>[".$value['id']."] ".$value['name']."</option>\r\n";
      }
    }

    // get data
    $this->offers_data_packets_arr = $this->cache_data_out("offers_data_packets_arr");
    // action)
    if ( !empty($this->offers_data_packets_arr) ) {
      foreach ($this->offers_data_packets_arr AS $key => $value) {
         if (!$value['column']) {
          echo  "<tr>";
          echo  "<td>".$value['name']."</td><td>&rarr;</td>";
          echo  "<td><select name='map_packets[".urlencode($key)."]'>";
          echo  "<option value='-1'>----------------------------------------</option>";
          echo  $price_col_options;
          echo  "</select></td>";
          echo "</tr>";
        }
      }
      // destroy variable
      unset ($this->offers_data_packets_arr);
    }
    echo "</table><br/>";

  }


  /**
    * This function get catalogue data
    * and put it in 2 arrays (catalogue_data_properties & catalogue_data_commodity)
    * @return true if arrays succesfully created
    */
  function get_catalogue_data ($plural_tag, $single_tag, $ignore_tags, $arr) {
    // walking...
    foreach ($arr AS $key => $value) {
      // catalogue property
      if ($value['n'] && $value['d']) {
        $this->catalogue_data_properties[$value['n']] = $value['d'];
      }

      // if plural tag - call recursive function
      if ( $value['n']==$plural_tag ) {
        $this->get_catalogue_data($plural_tag, $single_tag, $ignore_tags, $value['c']);
        continue;
      }

      // if single tag - put data in array
      if ( $value['n']==$single_tag ) {
        $index = count($this->catalogue_data_commodity);

        foreach ($value['c'] AS $child) {
          // put data in array
          if ( !in_array($child['n'], $ignore_tags) && $child['d'] ) {
            $this->catalogue_data_commodity[$index][$child['n']] = $child['d'];
            $this->catalogue_data_commodity_fields[$child['n']]++;
            continue;
          }
          // group(s)_id
          if ($child['n']=="Группы") {
            foreach ($child['c'] AS $group_data) {
              $this->catalogue_data_commodity[$index][$child['n']][] = array($group_data['n'] => $group_data['d']);
            }
            $this->catalogue_data_commodity_fields[$child['n']]++;
            continue;
          }
          // characteristics
          if ($child['n']=="ХарактеристикиТовара") {
            if ( !isset($this->catalogue_data_commodity_characteristics) ) $this->catalogue_data_commodity_characteristics = array();
            foreach ($child['c'] AS $characteristic_data) {
              $_name = "";
              $_value = "";
              foreach ($characteristic_data['c'] as $characteristic) {
                if ($characteristic['n']=="Наименование") $_name = $characteristic['d'];
                if ($characteristic['n']=="Значение") $_value = $characteristic['d'];
              }
              $this->catalogue_data_commodity[$index][$child['n']][$_name] = $_value;
              if ( !in_array($_name, $this->catalogue_data_commodity_characteristics) ) {
                $this->catalogue_data_commodity_characteristics[] = $_name;
              }
              $this->catalogue_data_commodity_fields[$_name]++;
            }
            continue;
          }
          ### // tax rates  // if ( $child['name']==$this->cu("СтавкиНалогов") ) {            // foreach ($child['children'] AS $tax_rates_data) {              // if ( $tax_rates_data['name']==$this->cu("СтавкаНалога") ) {                // $tax_rate_index = count($this->catalogue_data_commodity[$index][$child['name']]);                // $tax_name = "";                // $tax_value = 0;                // foreach ($tax_rates_data['children'] AS $tax_rate_data) {                  // if ( $tax_rate_data['name']==$this->cu("Наименование") ) $tax_name = $tax_rate_data['data'];                  // if ( $tax_rate_data['name']==$this->cu("Ставка") ) $tax_value = $tax_rate_data['data'];                // }                // //make array( 0 => array(tax_name => tax_value), 1 => ...)                // if ($tax_name && $tax_value) {                  // $this->catalogue_data_commodity[$index][$child['name']][$tax_rate_index] = array($tax_name => $tax_value);                // }              // }            // }          // }
        }
      }
    }
    // cache files
    $this->cache_data_in($this->catalogue_data_properties, "catalogue_data_properties");
    $this->cache_data_in($this->catalogue_data_commodity, "catalogue_data_commodity");
    $this->cache_data_in($this->catalogue_data_commodity_fields, "catalogue_data_commodity_fields");
    $this->cache_data_in($this->catalogue_data_commodity_characteristics, "catalogue_data_commodity_characteristics");
  }


  function import_catalogue_data () {
    // get data
    $this->catalogue_data_commodity = $this->cache_data_out("catalogue_data_commodity");
    $this->sub_structure = $this->cache_data_out("sub_structure");
    // return if no data found
    if ( empty($this->catalogue_data_commodity) ) return false;

    $map_field = array();
    // get data
    $this->catalogue_data_commodity_fields = $this->cache_data_out("catalogue_data_commodity_fields");
    $this->catalogue_data_commodity_characteristics = $this->cache_data_out("catalogue_data_commodity_characteristics");
    // action)
    if ( !empty($this->catalogue_data_commodity_fields) ) {
      $need_tags = array("Описание", "Статус", "Артикул", "Картинка", "Производитель");
      // добавляем характеристики как будто есть такие теги в родителе "Товар"
      // на самом деле есть только тэг "ХарактеристикиТовара"
      if ( isset($this->catalogue_data_commodity_characteristics) && is_array($this->catalogue_data_commodity_characteristics) ) {
        $need_tags = array_merge($need_tags, $this->catalogue_data_commodity_characteristics);
      }
      foreach ($this->catalogue_data_commodity_fields AS $xml_tag => $tag_count) {
        $xml_tag = trim($xml_tag);
        if ( !in_array($xml_tag, $need_tags) ) continue;

        $fields[$xml_tag]['name'] = $xml_tag;
        foreach ($this->shop_classes AS $class) {
          $map_field = $_POST['map_fields'][ $class['id'] ][ urlencode($xml_tag) ];

          // write compliance (second pass)
          if ($map_field) {
            $this->db->query("REPLACE INTO `Netshop_ImportMap`
              SET `source_id` = '".$this->source_id."',
              `type` = 'property',
              `source_string` = '".$this->db->escape($xml_tag)."',
              `value` = '".$this->db->escape($map_field)."'");
          }
        }

      }
      unset ($this->catalogue_data_commodity_fields);
    }

    if ( !empty($fields) ) {
      $res = $this->db->get_results("SELECT `source_string` AS id, `value`
        FROM `Netshop_ImportMap`
        WHERE `source_id` = '".$this->source_id."'
        AND `type` = 'property'
        AND `source_string` IN ('".join("','", array_keys($fields))."')", ARRAY_A);
      $fields_from_base = $this->db->num_rows;
    }

    if ( !empty($res) ) {
      foreach ($res AS $value) {
        $fields[ $value['id'] ]['column'] = $value['value'];
      }
    }

    $this->not_mapped_fields = count($fields) - $fields_from_base;
    // for map_fields_dialog function
    $this->not_mapped_fields_arr = is_array($this->not_mapped_fields_arr) ? array_merge($this->not_mapped_fields_arr, $fields) : $fields;

    //if ($this->not_mapped_fields) return false;

    // clear?
    if ($this->not_mapped_fields || !$this->everything_clear || !$this->sub_structure) return false;

    // get ralation array sub_id - sub_class_id - class_id
    if (!$this->templates) $this->templates = $this->get_templates();

    $current_num = 1;
    $total_objects = count($this->catalogue_data_commodity);
    foreach ($this->catalogue_data_commodity AS $key => $value) {
      // progress bar
      if ($current_num==1) {
        echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_COMMODITIES_IN_CATALOGUE."</b><br/>\r\n";
        $this->progress_bar_show("commodity_progress");
        echo "<br/>\r\n";
      }

      // main XML values
      $this_units = isset($value["БазоваяЕдиница"]) ? $value["БазоваяЕдиница"] : false;
      $this_groups = $value["Группы"][0]["Ид"];
      $this_id = $value["Ид"];
      $this_name = trim( $value["Наименование"] );
      $this_characteristics = $value["ХарактеристикиТовара"];
      //$this_taxes = $value[$this->cu("СтавкиНалогов")];

      // insert unknown units in base
      if ($this_units && !$this->units[$this_units]) {
        $this->db->query("INSERT INTO `Classificator_ShopUnits` SET `ShopUnits_Name` = '".$this_units."'");
        $this->units[$this_units] = $this->db->insert_id;
      }

      // values for base
      $this_prop = array();
      // ("Name", "ItemID", "Price", "Currency", "ImportSourceID") imperative fields
      $this_prop["Checked"] = 1;
      $this_prop["Name"]  = $this_name;
      $this_prop["Units"] = $this->units[$this_units];
      $this_prop["ImportSourceID"] = $this->source_id;
      $this_prop["ItemID"] = $this_id;
      $this_prop["Subdivision_ID"] = $this->sub_structure[$this_groups]['Subdivision_ID'];
      // Class_ID
      $this_class = $this->templates[ $this_prop["Subdivision_ID"] ]["class_id"];
      // ignored group, disabled in "map_sections_dialog" dialog
      if ($this_prop["Subdivision_ID"]==-1) {
        // increment
        $current_num++;
        continue;
      }
      // executed after MySQL insert
      $this_prop["Sub_Class_ID"] = $this->sub_structure[$this_groups]['Sub_Class_ID'];
      // Sub_Class data from base
      if (!$this_prop["Sub_Class_ID"]) $this_prop["Sub_Class_ID"] = $this->templates[ $this_prop["Subdivision_ID"] ]["subclass_id"];
      // Priority
      $this_prop["Priority"] = (int)$this->db->get_var("SELECT MAX(`Priority`) + 1 FROM `Message".$this_class."` WHERE `Sub_Class_ID` = '".$this_prop["Sub_Class_ID"]."'");
      // find VAT
      //if ( !empty($this_taxes) ) {
      //  foreach ($this_taxes AS $tax_data) {
      //    if ( $tax_data[$this->cu("НДС")] ) $this_prop["VAT"] = $tax_data[$this->cu("НДС")];
      //  }
      //}

      // try to find goods with same Item ID
      $exist_id = $this->db->get_var("SELECT `Message_ID` FROM `Message".$this_class."`
        WHERE `ItemID` = '".$this_id."'
        AND `ImportSourceID` = '".$this->source_id."'");

      // additional fields compile
      $filetable_lastid = 0;
      if ( !empty($fields) )
      foreach ($fields AS $field_key => $field_data) {
        if ($field_data['column']==-1) continue;
        list($field_name, $field_type) = $this->db->get_row("SELECT `Field_Name`, `TypeOfData_ID` FROM `Field`
          WHERE `Field_ID` = '".(int)$field_data['column']."'", ARRAY_N);
        $xml_value = trim( $value[$field_key] );

        switch (true) {
          // в массиве $this->catalogue_data_commodity есть тэг "ХарактеристикиТовара"
          // но значение следует брать из массива $this_characteristics, т.к. тэг составной
          case is_array($this_characteristics) && array_key_exists($field_key, $this_characteristics):
            $this_prop[$field_name] = $this_characteristics[$field_key];
          break;
          // тэг "Картинка"
          case $field_type==6 && $field_key=="Картинка":
            /** filename similarly commodity ID ($this_id) */
            if( !$xml_value || !file_exists($GLOBALS['TMP_FOLDER'].$xml_value) ) continue;
            // image properties
            list($filewidth, $fileheight, $filetype, $fileattr) = getimagesize($GLOBALS['TMP_FOLDER'].$xml_value);
            $filetype = image_type_to_mime_type($filetype);
            $filename = basename($GLOBALS['TMP_FOLDER'].$xml_value);
            $filesize = filesize($GLOBALS['TMP_FOLDER'].$xml_value);
            $this_prop[$field_name] = $filename.":".$filetype.":".$filesize;
            //echo $this_prop[$field_name]."<hr>";
            $file_copy_path = $this_prop["Subdivision_ID"]."/".$this_prop["Sub_Class_ID"]."/";
            // md5 name with salt
            $uniq_file_name = md5( $filename.microtime().uniqid("netcat") );

            $exist_sql = $this->db->get_row("SELECT `ID`, `Real_Name`, `Virt_Name` FROM `Filetable`
              WHERE `Message_ID` = '".$exist_id."' AND `Field_ID` = '".(int)$field_data['column']."'", ARRAY_N);

            if($exist_sql) {
              list($_id, $_real_name, $_virt_name) = $exist_sql;
              $this->db->query("UPDATE `Filetable`
                SET `Real_Name` = '".$filename."', `Virt_Name` = '".$uniq_file_name."', `File_Type` = '".$filetype."', `File_Size` = '".$filesize."'
                WHERE `ID` = '".$_id."'");
              # Delete old file ...
              @unlink($GLOBALS['FILES_FOLDER'].$file_copy_path.$_virt_name);
            }
            else {
              $this->db->query("INSERT INTO `Filetable`
                (`Real_Name`, `File_Path`, `Virt_Name`, `File_Type`, `File_Size`, `Message_ID`, `Field_ID`)
                VALUES
                ('".$filename."', '/".$file_copy_path."', '".$uniq_file_name."', '".$filetype."', '".$filesize."', '0', '".(int)$field_data['column']."')");
              $filetable_lastid = $this->db->insert_id;
            }

            // create dirs
            if( !isset($GLOBALS['DIRCHMOD']) ) $GLOBALS['DIRCHMOD'] = 0777;
            @mkdir($GLOBALS['FILES_FOLDER'].$this_prop["Subdivision_ID"], $GLOBALS['DIRCHMOD']);
            @mkdir($GLOBALS['FILES_FOLDER'].rtrim($file_copy_path, "/"), $GLOBALS['DIRCHMOD']);
            // copy file
            @copy($GLOBALS['TMP_FOLDER'].$xml_value, $GLOBALS['FILES_FOLDER'].$file_copy_path.$uniq_file_name);
          break;
          default:
            $this_prop[$field_name] = $xml_value;
        }
      }


      // collect fields in temp array
      $query = array();
      foreach ($this_prop AS $k => $v) {
        $query[] = "`".$k."` = '".$this->db->escape($v)."'";
      }
      // equip MySQL append query
      $query_str = "SET ".join(", ", $query);
      unset ($query);

      // create new goods or update existed
      if (!$exist_id) {
        $this->db->query("INSERT INTO `Message".$this_class."` ".$query_str.", `Created` = NOW()");
        $message_id = $this->db->insert_id;
        if ($filetable_lastid) $this->db->query("UPDATE `Filetable` SET `Message_ID` = '".(int)$message_id."' WHERE `ID` = '".(int)$filetable_lastid."'");
      }
      else {
        $this->db->query("UPDATE `Message".$this_class."` ".$query_str." WHERE `Message_ID` = '".$exist_id."'");
      }

      // procents completed
      $percent = intval($current_num/$total_objects * 100);
      $this->progress_bar_update("commodity_progress", $percent);
      // increment
      $current_num++;
    }
    // destroy variable
    unset ($this->catalogue_data_commodity);

  }


  function map_fields_dialog () {

    if ($silent_1c_import) return false;

    $this->everything_clear = false;

    echo "<b>".NETCAT_MODULE_NETSHOP_IMPORT_FIELDS_AND_TAGS_COMPLIANCE."</b>\r\n";
    echo "<table border='0' cellspacing='8' cellpadding='0'>\r\n";

    //all fields: Name, Description, Details, ItemID, Currency, Price, PriceMinimum, Image, Units, Vendor, StockUnits, ImportSourceID, CurrencyMinimum, TopSellingMultiplier, TopSellingAddition, VAT
    $exlude_fields_arr = array("Name", "ItemID", "Currency", "Price", "PriceMinimum", "ImportSourceID", "CurrencyMinimum", "TopSellingMultiplier", "TopSellingAddition", "VAT");

    // netshop goods classes
    foreach ($this->shop_classes AS $class) {

      $class_fields = $this->db->get_results("SELECT `Field_ID`, `Field_Name`, `Description`, `Class_ID` FROM `Field`
        WHERE `Class_ID` = '".$class['id']."'
        AND `Field_Name` NOT IN ('".join("', '", $exlude_fields_arr)."')", ARRAY_A);

      $fields_str = "";
      foreach ($class_fields AS $field) {
        $fields_str.= "<option value='".$field['Field_ID']."'>[".$field['Field_Name']."] - ".$field['Description']."</option>\r\n";
      }

      echo "<tr><td colspan='3' style='background:#EEE; padding:3px'>[".$class['id']."] ".$class['name']."</td></tr>";

      if ( !empty($this->not_mapped_fields_arr) ) {
        foreach ($this->not_mapped_fields_arr AS $key => $value) {
           if (!$value['column']) {
            echo "<tr>";
            echo "<td>".$value['name']."</td><td>&rarr;</td>";
            echo "<td><select name='map_fields[".$class['id']."][".urlencode($key)."]'>";
            echo "<option value='-1'>----------------------------------------</option>";
            echo $fields_str;
            echo "</select></td>";
            echo "</tr>";
          }
        }
      }

    }
    echo "</table><br/>";

  }

  /**
   * Show HTML progress bar
   * @param html id
   * @param return or echo
   * @return html text
   */
  function progress_bar_show ($html_id, $ret = false) {
    $result = "<div id='".$html_id."_line' style='position:absolute; border:1px solid #FFF; height:20px; width:0px; background:#5699c7;'></div>\r\n";
    $result.= "<div style='position:absolute; border:1px solid #333; text-align:center; height:20px; width:420px; background:none; color:#264863'><p id='".$html_id."_text' style='padding:0; margin:2px 0 0'>0%</p></div>\r\n";
    $result.= "<br clear='all'/>\r\n";
    if ($ret) return $result;
    echo $result;
  }

  /**
   * Update HTML progress bar
   * @param html id
   * @param percent
   * @param return or echo
   * @return html text
   */
  function progress_bar_update ($html_id, $percent, $ret = false) {
    $result = "<script type='text/javascript'>nc_netshop_import_progress('".$percent."', '".$html_id."');</script>\r\n";
    if ($ret) return $result;
    echo $result;
    flush();
  }


  /**
   * From UTF to Win
   */
  function uc ($text) { return $text;
    switch (true) {
      case extension_loaded("mbstring"):
        return mb_convert_encoding($text, "cp1251", "UTF-8");
      case extension_loaded("iconv"):
        return iconv("UTF-8", "cp1251", $text);
      default:
        return $this->uniconv->utf8ToStr($text);
    }
  }


  /**
   * From Win to UTF
   */
  function cu ($text) {
    switch (true) {
      case extension_loaded("mbstring"):
        return mb_convert_encoding($text, "UTF-8", "cp1251");
      case extension_loaded("iconv"):
        return iconv("cp1251", "UTF-8", $text);
      default:
        return $this->uniconv->strToUtf8($text);
    }
  }

}

?>