<?php

/* $Id: fragment.php 4352 2011-03-27 04:21:55Z kx $ */

/**
 *
 */
class nc_search_document_parser_html_fragment {

  protected $dom;

  /**
   * @param string $content
   */
  public function __construct($content = "") {
    $this->dom = new DOMDocument('1.0', 'UTF-8');
    // «правильный» подход с «более корректным» XML, неймспейсами и блудницами не сработал
    // (корёжит кодировку при импорте узлов несмотря на explicit указание кодировки)
    $this->set_html($content);
  }
  
  /**
   * @param string $html
   */
  protected function set_html($html) {
    // Для корректного парсинга нужен заголовок с указанием кодировки (указания
    // в конструкторе недостаточно). К счастью, DOMDocument великодушно прощает 
    // столь вольное обращение с HTML:
    $meta = "<meta http-equiv='content-type' content='text/html; charset=utf-8' />";
    $html .= "\n";
    $this->dom->loadHTML("$meta<nc-fragment>$html</nc-fragment>\n");
  }

  /**
   *
   * @param DOMNode $dest_node
   * @param DOMNodeList $imported_nodes
   */
  protected function append_nodes_to(DOMNode $dest_node, DOMNodeList $imported_nodes) {
    $dom = $this->dom;
    foreach ($imported_nodes as $node) {
      if ($node instanceof DOMAttr) {
        $dest_node->appendChild($dom->createTextNode($node->nodeValue . "\n"));
      }
      else {
        $dest_node->appendChild($dom->importNode($node, true));
        $dest_node->appendChild($dom->createTextNode("\n")); // добавить WHITESPACE
      }
    }
  }

  /**
   *
   * @param self $fragment
   */
  public function append_fragment(self $fragment) {
    $this->append_nodes_to($this->dom->documentElement, $fragment->get_nodes());
  }

  /**
   *
   * @param DOMNodeList $nodes
   */
  public function append_nodes(DOMNodeList $nodes) {
    $dom = $this->dom;
    $fragment_node = $dom->createElement('nc-fragment');
    $dom->appendChild($fragment_node);
    $this->append_nodes_to($fragment_node, $nodes);
  }

  /**
   * Получить узлы с контентом
   * @return DOMNodeList
   */
  public function get_nodes() {
    return $this->dom->getElementsByTagName('nc-fragment');
  }

  /**
   *
   * @return string
   */
  public function get_html() {
    preg_match_all("#<nc-fragment>(.*?)</nc-fragment>#s",
                    $this->dom->saveHTML(),
                    $matches);
    return trim(join("\n", $matches[1]));
  }

  /**
   *
   * @return string
   */
  public function get_text() {
    $text = trim($this->dom->textContent);
    // убрать лишние пустые строки:
    $text = strtr($text, "\r", "");
    $text = preg_replace("/(\s*\n\s*){3,}/u", "\n\n", $text);
    
    return $text;
  }

  /**
   * @param DOMNodeList $nodes
   * @param boolean $remove
   * @param nc_search_document_parser_html_part|null $recipient
   * @return boolean success
   */
  protected function extract_nodes($nodes, self $recipient = null, $remove = false) {
    if (!($nodes instanceof DOMNodeList) || !$nodes->length) { return false; }
    if ($recipient) { $recipient->append_nodes($nodes); } // must be done before removing the nodes ↓
    if ($remove) {
      $nodes_to_remove = array(); // надо делать в два этапа, нельзя удалять внутри foreach
      foreach ($nodes as $node) {
        if ($node instanceof DOMElement) { $nodes_to_remove[] = $node; }
        /* elseif ($node instanceof DOMAttr) { NOT IMPLEMENTED } */
      }
      foreach ($nodes_to_remove as $node) { $node->parentNode->removeChild($node); }
    }
    return true;
  }

  /**
   * @param string $xpath_query
   * @param nc_search_document_parser_html_part|null $recipient
   * @param boolean $remove
   * @return boolean success
   */
  public function extract_xpath($xpath_query, self $recipient = null, $remove = false) {
    return $this->extract_nodes($this->evaluate_xpath($xpath_query), $recipient, $remove);
  }

  /**
   * @param string $tag_name
   * @param nc_search_document_parser_html_part|null $recipient
   * @param boolean $remove
   * @return boolean success
   */
  public function extract_tag($tag_name, self $recipient = null, $remove = false) {
    return $this->extract_nodes($this->dom->getElementsByTagName($tag_name), $recipient, $remove);
  }

  /**
   * @param string $regexp
   * @param nc_search_document_parser_html_part|null $recipient
   * @param boolean $remove
   * @return boolean success
   */
  public function extract_regexp($regexp, self $recipient = null, $remove = false) {
    $html = $this->get_html();
    if (!nc_preg_match_all($regexp, $html, $matches)) { return false; }
    if ($recipient) { $recipient->append_fragment(new self(join("\n", $matches[1]))); }
    if ($remove) {
      // it's easier to dismiss the old DOMDocument than to delete all nodes properly
      $this->set_html(str_replace($matches[0], "", $html));
    }
    return true;
  }
  
  /**
   * 
   * @param string $xpath_query
   * @return DOMNodeList
   */
  public function evaluate_xpath($query) {
    $xpath = new DOMXPath($this->dom);
    return $xpath->evaluate($query);
  }

}
