<?php

/* $Id: collection.php 4330 2011-03-16 03:31:31Z kx $ */

/**
 * Список ссылок, обрабатываемых в текущей задаче.
 */
class nc_search_indexer_link_collection extends nc_search_data_persistent_collection {

  protected $data_class = 'nc_search_indexer_link';
  protected $index_by = 'url';
  
  protected $referrer_cache = array();

  /**
   *
   * @param array $urls
   * @param string $referrer
   * @return array array with link IDs
   */
  public function add_links(array $urls, $referrer = null) {
    $link_ids = array();
    foreach ($urls as $url) { $link_ids[] = $this->add_link($url, $referrer); }
    return $link_ids;
  }
  
  /**
   * Создает объект nc_search_indexer_link и добавляет его в коллекцию
   * @param string $url
   * @param string  $referrer
   * @return integer Link ID
   */
  public function add_link($url, $referrer = null) {
    $full_url = $this->normalize_url($this->resolve_link($url, $referrer));
    $full_url = nc_search_util::encode_url($full_url);

    if ($this->offsetExists($full_url)) {
      $link = $this->offsetGet($full_url);
    }
    else {
      $link = new nc_search_indexer_link(array("url" => $full_url));
      $link->save();
      $this->add($link);
      nc_search::log(nc_search::LOG_PARSER_DOCUMENT_LINKS, 
                     "Added link to the queue: " . nc_search_util::decode_url($full_url));
    }

    return $link->get_id();
  }
  
  /**
   * Получить абсолютный URL
   * @param string $url
   * @param string $referrer
   * @return string
   * @throws nc_search_exception
   */
  protected function resolve_link($url, $referrer = null) {
    // remove the fragment (hash) part of the link
    $url = strtok($url, "#");
    
    if (strpos(substr($url, 0, 12), "://")) { return $url; }
    
    if (!$referrer) {
      throw new nc_search_exception("Cannot resolve full URL: '$url' (no referrer)");
    }
    
    $base_url = parse_url($referrer);
    if ($url[0] != "/") { // relative url
      if (substr($base_url["path"], -1) != '/') {
        $base_url["path"] = dirname($base_url["path"]) . "/";
      }
      $path = $base_url["path"] . $url;
    }
    else { // absolute URL
      $path = $url;
    }

    // get rid of "./", "../"
    if (strpos($path, "./") !== false) {
      $path_parts = array();
      foreach (explode("/", $path) as $part) {
        if ($part == '.' || $part == '') { continue; }
        if ($part == '..') { array_pop($path_parts); }
        else { $path_parts[] = $part; }
      }
      $path = join("/", $path_parts);
      if ($path[0] != '/') { $path = "/$path"; }
    }

    $full_url = $base_url['scheme'] . "://" . 
                $base_url['host'] . 
                (isset($base_url["port"]) ? ":$base_url[port]" : "") . 
                $path;

    return $full_url;
  }

  /**
   * Производится сортировка параметров для того, чтобы не запрашивать страницу
   * дважды, если в ссылках на неё параметры перечислены в разном порядке, например:
   * /sub/?tag=22&curPos=10 и /sub/?curPos=10&tag=22 будут считаться одной страницей
   */
  protected function normalize_url($url) {
    if (!strpos($url, "&") || !nc_search::should('IndexerNormalizeLinks')) { return $url; }
    list($path, $params) = explode("?", $url);
    $params = explode("&", $params);
    sort($params);
    return $path . '?' . join("&", $params);
  }

  /**
   *
   * @return nc_search_indexer_link|null
   */
  public function get_next_link() {
    return $this->first('is_processed', false);
  }

  /**
   * For logging purposes
   */
  public function to_string() {
    $links = array();
    foreach (array_keys($this->items) as $url) {
      $links[] = nc_search_util::decode_url($url);
    }            
    return $links;
  }

}
