<?php

/* $Id: web.php 4334 2011-03-20 23:36:17Z kx $ */

/**
 *
 */
class nc_search_indexer_runner_web implements nc_search_indexer_runner {

  protected $cycle_number = 0;

  protected $delay;
  protected $memory_ratio;
  protected $memory_limit;
  protected $time_ratio;
  protected $time_limit;
  /**
   *
   */
  public function __construct() {
    $this->start_time = time();

    $this->time_ratio = nc_search::get_setting('IndexerTimeThreshold');
    $this->time_limit = ini_get('max_execution_time');
    $this->time_ratio = nc_search::get_setting('IndexerTimeThreshold');
    
    $this->memory_ratio = nc_search::get_setting('IndexerMemoryThreshold');
    $this->memory_limit = nc_search_util::int_from_bytes_string(ini_get('memory_limit'));

    $this->delay = nc_search::get_setting('CrawlerDelay');

    set_time_limit(0);
    ignore_user_abort(true);
  }
  
  /**
   * 
   */
  protected function interrupt_if_needed($indexer) {
    $memory_use = (function_exists('memory_get_usage')) ? memory_get_usage() : 0;
    $time_use = time() + $this->delay - $this->start_time;

    $out_of_memory = ($this->memory_limit > 0 && ($memory_use / $this->memory_limit) > $this->memory_ratio);
    $out_of_time = ($this->time_limit > 0 && ($time_use / $this->time_limit) > $this->time_ratio);

    if ($out_of_memory || $out_of_time) {
      $indexer->interrupt("mem: $memory_use bytes; time: $time_use s");
      return true;
    }
    
    return false;
  }
  

  /**
   *
   * @param nc_search_indexer $indexer
   * @return boolean is task finished
   */
  public function loop(nc_search_indexer $indexer) {
    $cycle_number = 0;
    $save_cycles = nc_search::get_setting('IndexerSaveTaskEveryNthCycle');
        
    while (true) {
      // stop prematurely:
      if (connection_status() !== CONNECTION_NORMAL) {
        $indexer->cancel();
        return true; // nobody listens anyway
      }

      if ($this->interrupt_if_needed($indexer)) { return false; }
      
      // сохранять задачу каждые X циклов
      if ($cycle_number % $save_cycles == 0) {
        $indexer->save_task();
      }
      
      switch ($indexer->next()) {
        case nc_search_indexer::TASK_FINISHED:
          return true; // we're done
        case nc_search_indexer::TASK_STEP_FINISHED:
          if ($this->delay) {
            if ($this->interrupt_if_needed($indexer)) { return false; }
            sleep($this->delay);
          }
          break;
        case nc_search_indexer::TASK_STEP_SKIPPED:
          break;
        default:
          throw new nc_search_exception("Incorrect return value from nc_search_indexer::next()");
      }
      
      $cycle_number++;
    }
  }

}
