<?php

/* $Id: morphy.php 4331 2011-03-16 03:34:08Z kx $ */

/**
 *
 */
class nc_search_language_analyzer_morphy extends nc_search_language_analyzer {

  /**
   * @var array of phpMorphy
   */
  static protected $instances = array();
  
  protected $max_cache_size = 10000;
  protected $cache = array(array(),array());

  /**
   *
   * @return phpMorphy
   */
  protected function get_morphy() {
    $language = $this->context->get('language');
    $language = $language . "_" . $language; // phpMorphy requires "ru_ru", "en_en"

    if (!isset(self::$instances[$language])) {
      if (!class_exists('phpMorphy', false)) {
        nc_search::load_3rdparty_script("phpmorphy/src/common.php");
      }

      $options = array(
        'storage' => PHPMORPHY_STORAGE_FILE,
        'predict_by_suffix' => true,
        'predict_by_db' => true,
      );

      // Path to directory where dictionaries are located
      $dict_path = nc_search::get_3rdparty_path() . '/phpmorphy/dicts';

      try {
        self::$instances[$language] = new phpMorphy($dict_path, $language, $options);
      }
      catch (phpMorphy_Exception $e) {
        throw new nc_search_exception("Error occured while creating phpMorphy instance: {$e->getMessage()}");
      }
    }

    return self::$instances[$language];
  }
  
  /**
   * 
   * @param string $word
   * @param boolean
   * @return false|string|array
   */
  protected function lemmatize($word, $predict) {
    $i = (int)$predict; // 0 = no prediction, 1 = with prediction
    if ($i==0 && isset($this->cache[1][$word])) { $i = 1; }
    if (!isset($this->cache[$i][$word])) {
      $morphy = $this->get_morphy();
      $type = ($predict ? phpMorphy::NORMAL : phpMorphy::IGNORE_PREDICT);
      $this->cache[$i][$word] = $morphy->getBaseForm($word, $type);
      
      if (sizeof($this->cache[$i]) > $this->max_cache_size) { 
          $this->cache[$i] = array_slice($this->cache[$i], -$this->max_cache_size/2); 
      }
    }
    return $this->cache[$i][$word];
  }

  /**
   *
   * @param array $terms
   * @return array
   */
  public function get_base_forms(array $terms) {
    $result = array();
    foreach ($terms as $term) {
      $base_form = $this->lemmatize($term, true);
      $result = array_merge($result, (array)($base_form ? $base_form : $term));
    }
    return array_unique($result);
  }

  /**
   *
   * @param array $terms
   * @return string
   */
  public function get_highlight_regexp(array $terms) {
    $morphy = $this->get_morphy();
    $result = array();
    foreach ($terms as $term) {
      $paradigms = $morphy->findWord($term);
      if ($paradigms) {
        foreach ($paradigms as $paradigm) {
          $result = array_merge($result, $paradigm->getAllForms());
        }
      }
      else {
        $result[] = $term;
      }
    }
    $regexp = "(" . join("|", array_unique($result)) . ")";
    return nc_search_util::word_regexp($regexp, "Si");
  }
  
  /**
   * Проверка слова по словарю
   * @param string  ВНИМАНИЕ, предполагается, что слово передано в корректном регистре (ЗАГЛАВНОМ)
   * @return boolean слово есть в словаре
   */
  public function check_word($word) {
    return ($this->lemmatize($word, false) != false);
  }

}
