<?php

/* $Id: analyzer.php 4331 2011-03-16 03:34:08Z kx $ */

/**
 * Анализатор для Zend_Search_Lucene, поведение которого определяется
 * настройками модуля поиска.
 *
 * Based on Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8(Num)
 */
class nc_search_provider_zend_analyzer extends Zend_Search_Lucene_Analysis_Analyzer_Common {

  protected $regexp_alnum = '/[\p{L}\p{N}]+/u'; // буквы+цифры
  protected $regexp_alpha = '/(?:\b(?:sub|site)\d+\b|[\p{L}]+)/u'; // буквы + идентификаторы разделов и сайтов
  protected $tokenizer_regexp;
  protected $token_stack = array();

  public function __construct() {
    // strip numbers?
    $this->tokenizer_regexp = nc_search::should('IgnoreNumbers') ? $this->regexp_alpha : $this->regexp_alnum;
  }

  /**
   * Reset token stream
   */
  public function reset() {
    $this->_position = 0;
    $this->_bytePosition = 0;
  }

  /**
   * Tokenization stream API
   * Get next token
   * Returns null at the end of stream
   *
   * @return Zend_Search_Lucene_Analysis_Token|null
   */
  public function nextToken() {
    if (sizeof($this->token_stack)) { return array_pop($this->token_stack); }
    if ($this->_input === null) { return null; }

    // внимание: функции работы со строками нельзя заменять на аналоги nc_*

    // It covers both cases a) there are no matches (preg_match(...) === 0)
    // b) error occured (preg_match(...) === FALSE)
    while (preg_match($this->tokenizer_regexp, $this->_input, $match, PREG_OFFSET_CAPTURE, $this->_bytePosition)) {

      // matched string
      $matched_word = $match[0][0];

      // binary position of the matched word in the input stream
      $bin_start_pos = $match[0][1];

      // character position of the matched word in the input stream
      $start_pos = $this->_position +
              mb_strlen(substr($this->_input,
                               $this->_bytePosition,
                               $bin_start_pos - $this->_bytePosition),
                       'UTF-8');
      // character postion of the end of matched word in the input stream
      $end_pos = $start_pos + mb_strlen($matched_word, 'UTF-8');

      $this->_bytePosition = $bin_start_pos + strlen($matched_word);
      $this->_position = $end_pos;

      $processed = $this->apply_nc_filters($matched_word);
      $count = sizeof($processed);
      if ($count > 0) {
        for ($i=1; $i < $count; $i++) { // i.e. if $count > 1
          $token = new Zend_Search_Lucene_Analysis_Token($processed[$i], $start_pos, $end_pos);
          // умная книга Lucene in Action советует установить $token->setPositionIncrement(0),
          // но, по-моему, разницы нет (в исходниках ZSL отмечено "todo: Process
          // $token->getPositionIncrement()" - может быть, в будущем заработает)
          //// $token->setPositionIncrement(0);
          $this->token_stack[] = $token;
        }
        return new Zend_Search_Lucene_Analysis_Token($processed[0], $start_pos, $end_pos);
      }
    } // end of while ()

    return null;
  }

  /**
   * Фильтры сделаны нестандартным для Lucene способом; оправдания следующие:
   *   - возможно, в будущем в модуле будет свой парсер, и он будет обрабатывать
   *     ситуации с синонимами и проч. переписыванием запросов, e.g.:
   *     (word1 word2) → (word1 word2 syn2); "word1 word2" → ("word1 word2" OR "word1 syn2")
   *   - чтобы сделать через addFilter(), нужно создать дублирующие классы или класс-прокси
   *     (или отказаться от идеи возможного использования фильтров netcat с
   *     другими поставщиками поиска, которые [*может быть*] будут)
   *   - стандартные фильтры создают множество временных объектов
   *     (каждый фильтр, если он что-то сделал, создает новый токен)
   *
   * @param string $token_text
   * @return array может содержать 0 (стоп-слова), 1 или несколько (синонимы) элементов
   *
   */
  protected function apply_nc_filters($term) {
    $context = nc_search::get_current_context();
    return nc_search_extension_manager::get('nc_search_language_filter', $context)
                ->stop_on(array())
                ->apply('filter', array($term));
  }

}
