<?php
/* $Id: nc_subdivision.class.php 4699 2011-05-17 14:16:25Z andrey $ */
if ( !class_exists("nc_System") ) die("Unable to load file.");

class nc_Subdivision extends nc_Essence {
  
  protected $db;
  
  /**
   * Constructor function
   */  
  public function __construct () {
    // load parent constructor
    parent::__construct();
    
    // system superior object
    $nc_core = nc_Core::get_object();
    // system db object
    if ( is_object($nc_core->db) ) $this->db = $nc_core->db;
    
  }

  public function get_by_id ($id, $item = "") {

    $res = array();
    if ( isset($this->data[$id])) $res = $this->data[$id];


    if ( empty($res) ) {
      $res = $this->db->get_row("SELECT * FROM `Subdivision` WHERE `Subdivision_ID` = '".intval($id)."'", ARRAY_A );
      if ( empty($res) ) {
        //return false;
        throw new Exception("Subdivision with id  ".$id." does not exist");
      }

      $page = nc_Core::get_object()->page;
      $lm_type           = $page->get_field_name('last_modified_type');
      $sm_field          = $page->get_field_name('sitemap_include');
      $sm_change_field   = $page->get_field_name('sitemap_changefreq');
      $sm_priority_field = $page->get_field_name('sitemap_priority');
      $lang_field        = $page->get_field_name('language');

      $this->data[$id] = $res;

      $real_value = array('Template_ID', 'DisallowIndexing', $lm_type, $sm_field, $sm_change_field, $sm_priority_field,
          $lang_field, 'Read_Access_ID', 'Write_Access_ID', 'Edit_Access_ID', 'Delete_Access_ID', 'Checked_Access_ID', 'Moderation_ID',
          'Cache_Access_ID', 'Cache_Lifetime');
      foreach ( $real_value as $v ) {
        $this->data[$id]['_db_'.$v] = $this->data[$id][$v];
      }
      $this->data[$id]['_nc_final'] = 0;

    }

    if ( !$this->data[$id]['_nc_final'] ) {
      $this->data[$id] = $this->convert_system_vars($this->data[$id]);
      $this->data[$id] = $this->inherit($this->data[$id]);
      $this->data[$id]['_nc_final'] = 1;
    }

    if ( $item ) {
      return array_key_exists($item, $this->data[$id]) ? $this->data[$id][$item] : "";
    }

    return $this->data[$id];
   }


   public function set_current_by_id ($id, $reset = false) {
     $nc_core = nc_Core::get_object();
     try {
       $this->current = $this->get_by_id($id, "", $reset);
     }
     catch ( Exception $e ) {
       $this->current = $this->get_by_id($nc_core->catalogue->get_current("Title_Sub_ID"));
     }

     // return result
     return $this->current;
    

  }


  /**
   * Получить информацию о разделе по url
   * @param string uri
   * @param int номер сайта, по умолчанию определяется по домену
   * @param string требуемый параметр, если не задан - функция возврщает массив
   * @return mixed
   */
  public function get_by_uri ( $uri, $catalogue = 0, $item = '' ) {
    $nc_core = nc_Core::get_object();
    // определение сайта
    if ( !$catalogue ) {
      $catalogue = $nc_core->catalogue->get_by_host_name($_SERVER['HTTP_HOST']);
      $catalogue = $catalogue['Catalogue_ID'];
    }

    $uri = rtrim(nc_substr( $uri, 0, strrpos($uri, '/') ),'/').'/';

    // find date in url
    nc_preg_match("|
        / (\d{4}) /
        (?:
          (\d{2}) /
        )?
        (?:
          (\d{2}) /
        )?
      |x", $uri, $regs);
    // date found --> crop date symbols
    if ($regs)  $uri = str_replace($regs[0], "/", $uri);

    // титульная страница
    if ( $uri == "/" || $uri == "" ) {
      $res = $this->get_by_id($nc_core->catalogue->get_by_id($catalogue, "Title_Sub_ID"));
    }

    // поиск в кэше
    if ( empty($res) &&  !empty($this->data) ) {
      foreach ( $this->data as $id => $values ) {
        if ( $values['Catalogue_ID'] == $catalogue  && $values['Hidden_URL'] == $uri) {
          $res = $this->data[$id];
        }
      }
    }

    // из базы
    if ( empty($res) ) {
      $res = $this->db->get_row("SELECT * FROM `Subdivision`
                                 WHERE `Catalogue_ID` = '".intval( $catalogue )."'
                                 AND `Hidden_URL` = '".$this->db->escape($uri)."'", ARRAY_A);
      if ( $res) {
        $res['_nc_final'] = 0;
        $this->data[$res['Subdivision_ID']] = $res;
      }
      else {
        $res = $this->get_by_id($nc_core->catalogue->get_current("E404_Sub_ID"));
      }
    }

    // processing system fields, inherit
    if (  !$res['_nc_final'] ) {
      $res = $this->get_by_id( $res['Subdivision_ID']);
    }

    if ( $item ) {
      return array_key_exists($item, $res) ? $res[$item] : "";
    }

     return $res;
  }


  /**
   * Get subdivision data by the URI parameter
   *
   * @param URI string
   * @param bool use returned value as current catalogue data
   * @param bool reset stored data in the static variable
   *
   * @return subdivision data associative array
   */
  public function set_current_by_uri ($reset = false) {
    // system superior object
    $nc_core = nc_Core::get_object();
    $catalogue = $nc_core->catalogue->get_current("Catalogue_ID");
   
    // URI
    $uri = $nc_core->url->get_parsed_url('path');

    $this->current = $this->get_by_uri($uri, $catalogue);
    return $this->current;
  }


  public function validate_hidden_url ($url) {
    // validate Hidden_URL
    return nc_preg_match("/^[\/_a-z".NETCAT_RUALPHABET."0-9-]+$/i", $url);
  }
  
  protected function inherit ($sub_env) {
    // system superior object
    $nc_core = nc_Core::get_object();
    $lm = $nc_core->page->get_field_name('last_modified_type');
    
    $sub = $sub_env["Subdivision_ID"];
    
    $level_count = $this->_level_count[$sub] = 0;
    $parent_tree = $this->_parent_tree[$sub][] = "";


    $parent_tree[$level_count] = $sub_env;
    $level_count++;

    $inherited_params = array('Read_Access_ID', 'Write_Access_ID', 'Edit_Access_ID','Checked_Access_ID',
      'Delete_Access_ID', 'Subscribe_Access_ID', 'Moderation_ID', $lm,
      $nc_core->page->get_field_name('sitemap_changefreq'),
      $nc_core->page->get_field_name('sitemap_priority'), $nc_core->page->get_field_name('language'));
    $inherited_params_minus = array($nc_core->page->get_field_name('sitemap_include'), 'DisallowIndexing');

    if ( $nc_core->modules->get_by_keyword("cache") ) {
      $inherited_params[] = 'Cache_Access_ID';
      $inherited_params[] = 'Cache_Lifetime';
    }

    $parent_sub = $sub_env["Parent_Sub_ID"];
    
    while ($parent_sub) {
      $parent_sub_env = $this->get_by_id($parent_sub);

      if (!$sub_env["Template_ID"]) {
        $sub_env["Template_ID"] = $parent_sub_env["Template_ID"];
        $sub_env["TemplateSettings"] = $parent_sub_env["TemplateSettings"];
      }
      

      foreach ( $inherited_params as $v ) {
        if (!$sub_env[$v]  ) $sub_env[$v] = $parent_sub_env[$v];
      }
      foreach ( $inherited_params_minus as $v ) {
        if (!$sub_env[$v] == -1  ) $sub_env[$v] = $parent_sub_env[$v];
      }


      $parent_tree[$level_count] = $parent_sub_env;
      $sub_env = $this->inherit_system_fields($this->essence, $parent_tree[$level_count], $sub_env);

      $level_count++;

      $parent_sub = $parent_sub_env["Parent_Sub_ID"];
    }

    $catalogue_env = $nc_core->catalogue->get_by_id($sub_env["Catalogue_ID"]);

    $sub_env = $this->inherit_system_fields("Catalogue", $catalogue_env, $sub_env);

    if (!$sub_env["Template_ID"]) {
     $sub_env["Template_ID"] = $catalogue_env["Template_ID"];
     $sub_env["TemplateSettings"] = $catalogue_env["TemplateSettings"];
    }

    foreach ( $inherited_params as $v ) {
      if (!$sub_env[$v] ) $sub_env[$v] = $catalogue_env[$v];
    }
    foreach ( $inherited_params_minus as $v ) {
      if ( $sub_env[$v] == -1 ) $sub_env[$v] = $catalogue_env[$v];
    }

    $parent_tree[$level_count] = $catalogue_env;
    $parent_tree[$level_count]["Subdivision_Name"] = $catalogue_env["Catalogue_Name"];
    $parent_tree[$level_count]["Hidden_URL"] = "/";
    
    // store data
    $this->_level_count[$sub] = $level_count;
    $this->_parent_tree[$sub] = $parent_tree;
    
    return $sub_env;
  }
  
  public function get_parent_tree ($sub) {
    // get data
    $this->get_by_id($sub);
    // check
    if ( is_array($this->_parent_tree[$sub]) && !empty($this->_parent_tree[$sub]) ) {
      return $this->_parent_tree[$sub];
    }
    // parent sub array not found
    return false;
  }
  
  public function get_level_count ($sub) {
    // get data
    $this->get_by_id($sub);
    // check
    if ( isset($this->_level_count[$sub]) ) {
      return $this->_level_count[$sub];
    }
    // level count array not found
    return false;
  }


  public function validate_english_name ($str) {
    // validate Hidden_URL
    return nc_preg_match("/^[a-z".NETCAT_RUALPHABET."0-9-]+$/i", $str);
  }



  public function get_lang ( $id ) {
    return $this->get_by_id($id, nc_Core::get_object()->page->get_language_field());
  }

   public function update ( $id, $params = array() ) {
    $db = $this->db;

    $id = intval($id);
    if ( !$id || !is_array($params) ) {
      return false;
    }

    $query = array();
    foreach ( $params as $k => $v ) {
      $query[] = "`".$db->escape($k)."` = '".$db->escape($v)."'";
    }

    if ( !empty($query) ) {
      $db->query("UPDATE `Subdivision` SET ".join(', ',$query)." WHERE `Subdivision_ID` = '".$id."' ");
      if ( $db->is_error )  throw new nc_Exception_DB_Error($db->last_query, $db->last_error);

      $this->core->event->execute("updateSubdivision", $this->get_by_id($id, 'Catalogue_ID'), $id);
    }

    $this->data = array();
    return true;
  }
}


?>
